@@ -61,6 +61,11 @@ import com.openexchange.tools.session.ServerSession;
  */
 public interface DriveSession {
 
+    /**
+     * The session parameter used to hold the client's push token
+     */
+    static final String PARAMETER_PUSH_TOKEN = "com.openexchange.drive.pushToken";
+
     /**
      * Gets the underlying server session.
      *
@@ -117,4 +122,18 @@ public interface DriveSession {
      */
     int getApiVersion();
 
+    /**
+     * Gets the client version.
+     *
+     * @return The client version
+     */
+    DriveClientVersion getClientVersion();
+
+    /**
+     * Gets the client type.
+     *
+     * @return The client type, or {@link DriveClientType#UNKNOWN} if not known
+     */
+    DriveClientType getClientType();
+
 }
