@@ -49,6 +49,7 @@
 
 package com.openexchange.drive.management;
 
+import java.util.Arrays;
 import java.util.EnumMap;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.regex.Pattern;
@@ -404,8 +405,11 @@ public class DriveConfig implements Initialization {
         /*
          * throttling
          */
-        maxBandwidth = configService.getIntProperty("com.openexchange.drive.maxBandwidth", -1);
-        maxBandwidthPerClient = configService.getIntProperty("com.openexchange.drive.maxBandwidthPerClient", -1);
+        String maxBandwidthValue = configService.getProperty("com.openexchange.drive.maxBandwidth");
+        maxBandwidth = Strings.isEmpty(maxBandwidthValue) || "-1".equals(maxBandwidthValue) ? -1 : parseBytes(maxBandwidthValue);
+        String maxBandwidthPerClientValue = configService.getProperty("com.openexchange.drive.maxBandwidthPerClient");
+        maxBandwidthPerClient = Strings.isEmpty(maxBandwidthPerClientValue) || "-1".equals(maxBandwidthPerClientValue) ?
+            -1 : parseBytes(maxBandwidthPerClientValue);
         maxConcurrentSyncOperations = configService.getIntProperty("com.openexchange.drive.maxConcurrentSyncOperations", -1);
         maxDirectoryActions = configService.getIntProperty("com.openexchange.drive.maxDirectoryActions", 1000);
         maxFileActions = configService.getIntProperty("com.openexchange.drive.maxFileActions", 500);
@@ -481,6 +485,40 @@ public class DriveConfig implements Initialization {
         }
     }
 
+    /**
+     * Parses a byte value including an optional unit.
+     *
+     * @param value the value to parse
+     * @return The parsed number of bytes
+     * @throws NumberFormatException If the supplied string is not parsable or greater then <code>Integer.MAX_VALUE</code>
+     */
+    private static int parseBytes(String value) throws NumberFormatException {
+        StringBuilder numberAllocator = new StringBuilder(8);
+        StringBuilder unitAllocator = new StringBuilder(4);
+        for (int i = 0; i < value.length(); i++) {
+            char c = value.charAt(i);
+            if (Character.isDigit(c) || '.' == c || '-' == c) {
+                numberAllocator.append(c);
+            } else if (false == Character.isWhitespace(c)) {
+                unitAllocator.append(c);
+            }
+        }
+        double number = Double.parseDouble(numberAllocator.toString());
+        if (0 < unitAllocator.length()) {
+            String unit = unitAllocator.toString().toUpperCase();
+            int exp = Arrays.asList("B", "KB", "MB", "GB").indexOf(unit);
+            if (0 <= exp) {
+                number *= Math.pow(1024, exp);
+            } else {
+                throw new NumberFormatException(value);
+            }
+        }
+        if (Integer.MAX_VALUE >= number) {
+            return (int)number;
+        }
+        throw new NumberFormatException(value);
+    }
+
     /**
      * Gets the (soft) minimum version limit for the supplied client type
      *
